/******
 *
 * Arquivo Pilha2.c: Implementao do mdulo Pilha
 *
 *****/

/*********************** Includes *************************/

#include <stdlib.h> /* Funo exit()   */
#include <stdio.h>  /* Funo printf() */
#include <string.h> /* strcpy() e strchr() */
#include <time.h>   /* time() e difftime() */

/****************** Constantes Simblicas *****************/

   /* Nmero mximo de arquivos temporrios gerados */
#define MAX_ARQ_TMP 99

   /* String com que inicia o nome    */
   /* de um arquivo temporrio gerado */
#define INI_ARQ_TMP "_tmp"

   /* Tamanho mximo de um nome de */
   /* arquivo  temporrio gerado   */
#define MAX_NOME_TMP 10

/****************** Definies de Macros ******************/

   /* Macro utilizada em tratamento */
   /* de condies de exceo       */
#define ASSEGURA(condicao, msg) if (!(condicao)) {\
                                   fprintf(stderr, "\n%s\n", msg);\
                                   exit(1); \
                                }

/****************** Definies de Tipos *******************/

   /* Tipo de cada arquivo temporrio */
typedef struct {
           char  nome[MAX_NOME_TMP]; /* Nome do arquivo */
           FILE *stream; /* Stream associado ao arquivo */
        } tArquivo;

typedef tArquivo tElemento;

   /* Tipo de n e tipo de ponteiro para n de uma */
   /* pilha encadeada de arquivos temporrios      */
typedef struct rotPilha {
           tElemento        conteudo;
           struct rotPilha *proximo;
        } tNo, *tPilha;

/******************* Aluses de Funes *******************/

extern void CriaPilha(tPilha *p);
extern void DestroiPilha(tPilha *pilha);
extern int PilhaVazia(const tPilha *p);
extern tElemento ElementoTopo(const tPilha *p);
extern void Empilha(tPilha *p, const tElemento *item);
extern tElemento Desempilha(tPilha *p);

/**************** Definies de funes ******************/

/****
 *
 * CriaPilha(): Inicializa uma pilha
 *
 * Parmetros:
 *      *p (sada) - a pilha que ser iniciada
 *
 * Retorno: Nada
 *
 ****/
void CriaPilha(tPilha *p)
{
      /* Torna nulo o ponteiro para o topo da pilha, que  o   */
      /* primeiro elemento de uma lista simplesmente encadeada */
   *p = NULL;
}

/****
 *
 * DestroiPilha(): Libera o espao ocupado pela pilha
 *
 * Parmetros: pilha (entrada/sada) - a pilha
 *
 * Retorno: Nada
 *
 ****/
void DestroiPilha(tPilha *pilha)
{
   tPilha p; /* Aponta para o prximo n a ser liberado */

   if (!*pilha) { /* Verifica se a pilha est vazia */
      return; /* Lista vazia no precisa ser destruda */
   }

   p = *pilha; /* Faz p apontar para o incio da pilha */

      /* Visita cada n da pilha liberando-o */
   do {
         /* Passa para o prximo n antes que o n corrente */
         /* seja destrudo. No importa que o ponteiro para */
         /*o incio da pilha seja alterado. Afinal, este    */
         /* ponteiro ser mesmo anulado.                    */
      *pilha = (*pilha)->proximo;

      free(p); /* Libera o espao do n corrente */

         /* Faz p apontar para o prximo n, cujo */
         /* endereo est armazenado em *pilha    */
      p = *pilha;
   } while (p);

      /* Neste ponto, a pilha ficou vazia, pois o  */
      /* ltimo valor assumido por *pilha foi NULL */
}

/****
 *
 * PilhaVazia(): Verfica se uma pilha est vazia
 *
 * Parmetros:
 *      p (entrada) - ponteiro para a pilha
 *
 * Retorno: 1 se a pilha recebida como parmetro estiver vazia;
 *          0, em caso contrrio
 *
 ****/
int PilhaVazia(const tPilha *p)
{
      /* A pilha est vazia quando o ponteiro */
      /* que a representa  NULL              */
   return !*p;
}

/****
 *
 * ElementoTopo(): Obtm o elemento do topo de uma pilha
 *
 * Parmetros:
 *      p (entrada) - ponteiro para a pilha
 *
 * Retorno: O elemento do topo da pilha recebida como parmetro
 *
 * Observao: Se a pilha estiver vazia, causa aborto de programa
 *
 ****/
tElemento ElementoTopo(const tPilha *p)
{
      /* Se a pilha estiver vazia, aborta o programa */
   ASSEGURA(!PilhaVazia(p), "A pilha esta' vazia");

   return (*p)->conteudo;
}

/****
 *
 * Empilha(): Empilha um item numa pilha
 *
 * Parmetros:
 *      p (entrada e sada) - ponteiro para a pilha na qual
 *                            ocorrer o empilhamento
 *      item (entrada) - ponteiro para o contedo
 *                       que ser empilhado
 *
 * Retorno: Nada
 *
 * Observao: Se a pilha estiver cheia, causa aborto
 *
 ****/
void Empilha(tPilha *p, const tElemento *item)
{
   tPilha pNovoNo;

      /* Tenta alocar um n da pilha */
   pNovoNo = malloc(sizeof(tPilha *)); /* Bug!!!! */

      /* Deveria ser: */
   pNovoNo = malloc(sizeof(tNo));

      /* Se a alocao no foi possvel aborta o programa */
   ASSEGURA(pNovoNo, "Nao houve alocacao");

   /*                                                    */
   /* O topo da pilha  o primeiro elemento de uma lista */
   /* encadeada. Logo, empilhar  o mesmo que inserir    */
   /* um elemento no incio de uma lista encadeada.      */
   /*                                                    */

      /* Armazena o contedo do novo n */
   pNovoNo->conteudo = *item;

      /* Faz o novo n apontar para */
      /* o topo corrente da pilha   */
   pNovoNo->proximo = *p;

      /* O novo n passa a ser o topo */
   *p = pNovoNo;
}

/****
 *
 * Desempilha(): Desempilha o elemento do topo de uma pilha
 *
 * Parmetros:
 *      p (entrada e sada ) - ponteiro para a pilha na qual
 *                             ocorrer o desempilhamento
 *
 * Retorno: Elemento do topo da pilha apontada por p
 *
 * Observao: Se a pilha estiver vazia, causa aborto
 *
 ****/
tElemento Desempilha(tPilha *p)
{
   tPilha    aux; /* Ponteiro que auxilia a operao */
   tElemento conteudo;

      /* Se a pilha estiver vazia, aborta o programa */
   ASSEGURA(!PilhaVazia(p), "A pilha esta' vazia");

   /*                                                    */
   /* O topo da pilha  o primeiro elemento de uma lista */
   /* encadeada. Logo, empilhar  o mesmo que remover o  */
   /* elemento inicial de uma lista encadeada.           */
   /*                                                    */

      /* Armazena o contedo do elemento do topo */
   conteudo = (*p)->conteudo;

      /* Faz 'aux' apontar para o topo da pilha */
   aux = *p;

      /* Faz com que o topo passe a ser */
      /* o segundo elemento da lista    */
   *p = (*p)->proximo;

      /* Libera o n que constitua o antigo topo */
   free(aux);

      /* Retorna o valor do segundo parmetro */
   return conteudo;
}

/****
 *
 * main(): Cria nomes de arquivos temporrios e tenta abri-los
 *         simultaneamente
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(void)
{
   tPilha p;

   printf("\nsizeof(tPilha *) = %d\n", sizeof(tPilha *));
   printf("\nsizeof(*p) = %d\n", sizeof(*p));
   printf("\nsizeof(tPilha) = %d\n", sizeof(tPilha));
   printf("\nsizeof(tNo) = %d\n", sizeof(tNo));
   printf("\nsizeof(tArquivo) = %d\n", sizeof(tArquivo));
   printf("\nsizeof(tElemento) = %d\n", sizeof(tElemento));

   return 0;
}

/******************* Exemplo de Execuo *******************

sizeof(tPilha *) = 4

sizeof(tPilha) = 4

sizeof(tNo) = 20

sizeof(tArquivo) = 16

sizeof(tElemento) = 16

***********************************************************/
