/****
 *
 * Ttulo: Implementao de Pilhas Usando Arrays Dinmicos
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Operaes sobre pilhas implementadas com arrays
 *            dinmicos
 *
 ****/

/*********************** Includes *************************/

#include <string.h>   /* Funes strXXX()        */
#include <stdlib.h>   /* malloc()                */
#include "ASSEGURA.h" /* Macro ASSEGURA()        */
#include "pilha.h"    /* Interface do tipo pilha */

/***************** Constantes Simblicas ******************/

   /* Nmero de elementos que sero acrescentados ao */
   /* array a cada alocao ou realocao de memria */
#define ACRESCIMO 10

/***************** Definies de Funes ******************/

/****
 *
 * CriaPilha(): Cria uma pilha
 *
 * Parmetros:
 *     umaPilha (sada) - ponteiro para a pilha que ser criada
 *
 * Retorno: Nada
 *
 ****/
void CriaPilha(tPilha **umaPilha)
{
      /* Aloca espao para a estrutura que armazena a pilha */
   *umaPilha = malloc(sizeof(tPilha));

      /* Garante que a alocao realmente ocorreu */
   ASSEGURA(umaPilha, "Nao foi possivel alocar pilha");

   (*umaPilha)->topo = -1; /* Inicia o topo da pilha */

      /* Aloca um array inicial */
   (*umaPilha)->itens = calloc(ACRESCIMO, sizeof(tElemento));

      /* Garante que o array foi realmente alocado */
   ASSEGURA( (*umaPilha)->itens, "Nao foi possivel alocar "
             "array para conter elementos da pilha" );

      /* Atualiza a varivel que armazena o tamanho do array */
   (*umaPilha)->tamanhoArray = ACRESCIMO;
}

/****
 *
 * DestroiPilha(): Libera o espao ocupado pela pilha
 *
 * Parmetros: pilha (entrada e sada) - a pilha
 *
 * Retorno: Nada
 *
 ****/
void DestroiPilha(tPilha *pilha)
{
      /* Se a pilha no estiver vazia, libera o espao */
      /* ocupado pelo array que contm os elementos    */
   if (pilha->topo) {
      free(pilha->itens);
   }

      /* Libera a estrutura que representa a pilha */
   free(pilha);
}

/****
 *
 * PilhaVazia(): Verfica se uma pilha est vazia
 *
 * Parmetros:
 *      p (entrada) - ponteiro para a pilha
 *
 * Retorno: 1 se a pilha recebida como parmetro estiver vazia;
 *          0, em caso contrrio
 *
 ****/
int PilhaVazia(const tPilha *p)
{
   return p->topo == -1;
}

/****
 *
 * ElementoTopo(): Obtm o elemento do topo de uma pilha
 *
 * Parmetros:
 *      p (entrada) - ponteiro para a pilha
 *
 * Retorno: O elemento do topo da pilha recebida como parmetro
 *
 * Observao: Se a pilha estiver vazia, causa aborto de programa
 *
 ****/
tElemento ElementoTopo(const tPilha *p)
{
   ASSEGURA(!PilhaVazia(p), "Erro: Pilha vazia.");

   return p->itens[p->topo];
}

/****
 *
 * Empilha(): Empilha um item numa pilha
 *
 * Parmetros:
 *      p (entrada e sada) - ponteiro para a pilha na qual
 *                            ocorrer o empilhamento
 *      item (entrada) - ponteiro para o contedo
 *                       que ser empilhado
 *
 * Retorno: Nada
 *
 * Observao: Se a pilha estiver cheia, causa aborto
 *
 ****/
void Empilha(tPilha *p, const tElemento *item)
{
   tElemento *novoArray; /* Ponteiro para o array */
                         /* redimensionado        */

      /* Se o array estiver repleto, tenta redimension-lo */
   if (p->topo >= p->tamanhoArray) {
         /* Calcula o novo tamanho do array */
      p->tamanhoArray += ACRESCIMO;

         /* Tenta redimensionar o array */
      novoArray = realloc(p->itens,
                          p->tamanhoArray*sizeof(tElemento));

         /* Checa se houve realocao de memria */
      ASSEGURA(novoArray, "Nao houve redimensionamento");

         /* O ponteiro que apontava para o incio */
         /* do array pode no ser mais vlido.    */
         /* Portanto,  preciso atualiz-lo.      */
      p->itens = novoArray;
   }

   p->itens[++p->topo] = *item;  /* Efetua o empilhamento */
}

/****
 *
 * Desempilha(): Desempilha o elemento do topo de uma pilha
 *
 * Parmetros:
 *      p (entrada e sada ) - ponteiro para a pilha na qual
 *                              ocorrer o empilhamento
 *
 * Retorno:  elemento do topo da pilha apontada por p
 *
 * Observao: Se a pilha estiver vazia, causa aborto
 *
 ****/
tElemento Desempilha(tPilha *p)
{
   ASSEGURA(!PilhaVazia(p), "Erro: Pilha vazia.");

   return p->itens[p->topo--];
}
