/****
 *
 * Ttulo: Lista de Compras
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Gerencia uma lista de compras com um menu de opes
 *            sensvel ao contexto
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Entrada e sada          */
#include <string.h>   /* Processamento de strings */
#include "Lista.h"    /* Interface do mdulo      */
#include "ASSEGURA.h" /* Macro ASSEGURA           */

/***************** Definies de Funes ******************/

/****
 *
 * EstaCheia(): Verifica se uma lista ordenada est cheia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver cheia; 0, em caso contrario
 *
 ****/
int EstaCheia(const tLista *lista)
{
   return lista->nElementos >= MAX_ELEMENTOS;
}

/****
 *
 * IniciaLista(): Inicia uma lista ordenada
 *
 * Parmetros: lista (sada) - a lista que ser iniciada
 *
 * Retorno: Nada
 *
 ****/
void IniciaLista(tLista *lista)
{
   lista->nElementos = 0;
}

/****
 *
 * Comprimento(): Calcula o comprimento de uma lista
 *
 * Parmetros: *lista (entrada) - a lista cujo comprimento
 *                                ser calculado
 *
 * Retorno: O nmero de elementos na lista
 *
 ****/
int Comprimento(const tLista *lista)
{
   return lista->nElementos; /* No h o que calcular */
}

/****
 *
 * ObtemElemento(): Obtm o elemento de uma lista que se
 *                  encontra numa dada posio
 *
 * Parmetros:
 *     *lista (entrada) - a lista
 *      indice (entrada) - o elemento
 *
 * Retorno: o nmero de elementos na lista
 *
 ****/
char *ObtemElemento(const tLista *lista, int pos)
{
   ASSEGURA( pos >= 0 && pos < lista->nElementos,
             "Elemento inexistente" );

   return (char *) lista->elementos[pos];
}

/****
 *
 * InsereElemento(): Insere um novo elemento numa dada posio
 *                   de uma lista ordenada
 *
 * Parmetros:
 *      lista (entrada/sada) - lista na qual ser feita a
 *                              insero
 *      elemento (entrada) - valor do elemento que ser
 *                           inserido
 *      pos (entrada) - a posio de insero
 *
 * Retorno: Nada
 *
 ****/
void InsereElemento(tLista *lista, tElemento elemento, int pos)
{
   ASSEGURA(!EstaCheia(lista), "A lista esta cheia");

      /* Abre espao para o novo elemento */
   for (int i = lista->nElementos; i >= pos; --i) {
      strcpy( (char *)lista->elementos[i + 1],
              (char *)lista->elementos[i] );
   }

      /* Insere o novo elemento */
   strcpy((char *)lista->elementos[pos], (char *)elemento);

   ++lista->nElementos; /* O tamanho da lista aumentou */
}

/****
 *
 * RemoveElemento(): Remove um elemento numa dada posio
 *                   de uma lista ordenada
 *
 * Parmetros:
 *      *lista (entrada/sada) - lista onde ser feita a
 *                               remoo
 *       pos (entrada) - posio do elemento que ser removido
 *
 * Retorno: Endereo do item removido
 *
 ****/
char *RemoveElemento(tLista *lista, int pos)
{
   int i;
   static tElemento itemRemovido;

   ASSEGURA(!EstaVazia(lista), "A lista esta' vazia");

   strcpy( (char *)itemRemovido, (char *)lista->elementos[pos] );

      /* Remover um elemento significa mover cada elemento  */
      /* uma posio para trs a partir do prximo elemento */
      /* adiante daquele que ser  removido                 */
   for (i = pos; i < lista->nElementos - 1; i++)
      strcpy( (char *)lista->elementos[i],
              (char *)lista->elementos[i + 1] );

   --lista->nElementos; /* O tamanho da lista diminuiu */

   return (char *)itemRemovido;
}

/****
 *
 * EstaVazia(): Verifica se uma lista ordenada est vazia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver vazia; 0, em caso contrario
 *
 ****/
int EstaVazia(const tLista *lista)
{
   return lista->nElementos == 0;
}

/****
 *
 * BuscaElemento(): Procura um elemento numa lista que
 *                  apresenta um determinado valor
 *
 * Parmetros:
 *      *lista (entrada) - lista na qual ser efetuada a busca
 *       elemento (entrada) - valor do elemento que ser
 *                            procurado
 *
 * Retorno: ndice do elemento se ele for encontrado;
 *          -1 em caso contrrio
 *
 ****/
int BuscaElemento(const tLista *lista, tElemento elemento)
{
   for (int i = 0; i < lista->nElementos; ++i) {
      if (!strcmp(lista->elementos[i], elemento)) {
         return i; /* Elemento foi encontrado */
      }
   }

   return -1; /* Elemento no foi encontrado */
}

/****
 *
 * AcrescentaElemento(): Acrescenta um novo elemento ao final
 *                       de uma lista ordenada
 *
 * Parmetros:
 *      *lista (sada) - lista onde ser feita o acrscimo
 *       elemento (entrada) - valor do elemento que ser
 *                            acrescentado
 *
 * Retorno: Nada
 *
 ****/
void AcrescentaElemento(tLista *lista, tElemento elemento)
{
   ASSEGURA(!EstaCheia(lista), "A lista esta cheia");

      /* Acrescenta um novo elemento ao final da lista */
   strcpy( (char *)lista->elementos[lista->nElementos],
           (char *)elemento );

   ++lista->nElementos; /* O tamanho da lista aumentou */
}

/****
 *
 * AlteraElemento(): Altera o valor de um elemento que se
 *                   encontra num determinada posio de
 *                   uma lista sem ordenao
 *
 * Parmetros:
 *      *lista (entrada e sada) - lista na qual o elemento se encontra
 *       elemento (entrada) - valor do elemento que ser
 *                            acrescentado
 *
 * Retorno: 0, se a operao for bem sucedida;
 *          1, em caso contrrio
 *
 ****/
void AlteraElemento(tLista *lista, tElemento valor, int pos)
{
   ASSEGURA( pos >= 0 && pos < lista->nElementos,
             "Elemento inexistente" );

   strcpy((char *)lista->elementos[pos], (char *)valor);
}
