/****
 *
 * Ttulo: Checando rvores Binrias de Busca
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 21/03/2017
 * ltima modificao: 21/03/2017
 *
 * Descrio: Verifica se uma rvore binria 
 *            uma rvore binria de busca
 *
 * Entrada: No h
 *
 * Sada: Uma mensagem indicando se a rvore binria criada
 *        pelo programa  uma rvore binria DE BUSCA
 *
 ****/

/*********************** Includes *************************/

#include<stdio.h>
#include<stdlib.h>

/****************** Definies de Tipos *******************/

typedef int tConteudo;

   /* Tipo de n e tipo de ponteiro  */
   /* para n de uma rvore binria  */
typedef struct no {
           struct no *esquerda;
           tConteudo  conteudo;
           struct no *direita;
        } tNo, *tArvoreBB;

typedef int (*tCompara) (const void *, const void *);

/***************** Definies de Funes ******************/

/****
 *
 * EhArvoreDeBusca(): Verifica se uma rvore binria  uma
 *                    rvore binria DE BUSCA
 *
 * Parmetros:
 *      p (entrada) - ponteiro para a raiz da rvore
 *      compara (entrada) - ponteiro para uma funo que
 *                          compara contedos dos ns de
 *                          acordo com o mesmo critrio de
 *                          comparao de qsort()
 *
 * Retorno: 1, se a rvore for uma rvore binria de busca;
 *          0, em caso contrrio
 *
 ****/
int EhArvoreDeBusca(tNo *p, tCompara compara)
{
   static tNo *antecessor = NULL; /* Armazenar o antecessor */
                                  /* de cada n visitado em  */
                                  /* ordem infixa            */

      /* Efetua um caminhamento em ordem infixa guardando */
      /* o valor do n antecessor imediato do n corrente */
   if (p) {
         /* Verifica se a subrvore esquerda  de busca */
      if (!EhArvoreDeBusca(p->esquerda, compara))
         return 0;

         /* Se o contedo do n corrente for menor do */
         /* que o contedo do n antecessor, no se   */
         /* trata de rvore de busca                  */
      if ( antecessor &&
           compara(&p->conteudo, &antecessor->conteudo) < 0 )
         return 0;

         /* O antecessor passa a ser o n corrente */
      antecessor = p;

         /* Verifica se a subrvore direita  de busca */
      return EhArvoreDeBusca(p->direita, compara);
   }

      /* Se ainda no houve retorno, a rvore foi aprovada */
   return 1;
}

/****
 *
 * DestroiArvoreBin(): Destri uma rvore binria
 *
 * Parmetros:
 *      arvore (entrada/sada) - ponteiro para a raiz
 *                               da rvore binria
 *
 * Retorno: Nada
 *
 ****/
void DestroiArvoreBin(tNo *arvore)
{
   if (arvore) {
         /* Destri subrvore esquerda */
      DestroiArvoreBin(arvore->esquerda);

         /* Destri subrvore direita */
      DestroiArvoreBin(arvore->direita);

         /* Remove a raiz */
      free(arvore); /* Libera a raiz */
   }
}

/****
 *
 * Comparacao(): Compara dois valores inteiros
 *
 * Parmetros:
 *      a, b (entrada) - valores inteiros a ser comparados
 *
 * Retorno:
 *    > 0, se a > b
 *    0, a = b
 *    < 0, a < b
 *
 ****/
int Comparacao(const void *a, const void *b)
{
   return *(int *)a - *(int *)b;
}

/****
 *
 * NoNovo(): Cria um n de uma rvore binria com o contedo
 *           especificado e subrvores vazias
 *
 * Parmetros:
 *      conteudo (entrada) - o contedo efetivo do n
 *
 * Retorno: Endereo do n criado
 *
 ****/
tNo *NoNovo(tConteudo conteudo)
{
   tNo *pNo = malloc(sizeof(tNo));

      /* Verifica se o n foi alocado */
   if (!pNo) {
      printf("\nUm no' da arvore nao foi alocado\n");
      exit(1);
   }

      /* Recheia o n */
   pNo->conteudo = conteudo;
   pNo->esquerda = pNo->direita = NULL;

   return pNo;
}

/****
 *
 * main(): Cria duas rvores binrias e verifica se elas so
 *         rvores binrias de busca
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   tNo *raiz = NoNovo(12); /* Raiz da rvore da Figura (a) */

      /* Insere os demais ns na rvore da Figura (a) */
   raiz->esquerda = NoNovo(10);
   raiz->direita = NoNovo(13);
   raiz->esquerda->esquerda = NoNovo(7);
   raiz->esquerda->direita = NoNovo(11);
   raiz->esquerda->esquerda->esquerda = NoNovo(5);

      /* Testa se a rvore binria da Figura (a)  de busca */
   if (EhArvoreDeBusca(raiz, Comparacao))
      printf("\n\t>>> A arvore da Figura (a) e' de busca");
   else
      printf("\n\t>>> A arvore da Figura (a) NAO e' de busca");

      /* Destri a rvore da Figura (a) */
   DestroiArvoreBin(raiz);

      /* Cria a rvore da Figura (b) */
   raiz = NoNovo(12);
   raiz->esquerda = NoNovo(10);
   raiz->direita = NoNovo(13);
   raiz->esquerda->esquerda = NoNovo(7);
   raiz->esquerda->direita = NoNovo(15);

      /* Testa se a rvore binria da Figura (b)  de busca */
   if (EhArvoreDeBusca(raiz, Comparacao))
      printf("\n\t>>> A arvore da Figura (b) e' de busca");
   else
      printf("\n\t>>> A arvore da Figura (b) NAO e' de busca");

   return 0;
}

/****************** Resultado do Programa *****************

        >>> A arvore da Figura (a) e' de busca
        >>> A arvore da Figura (b) NAO e' de busca

**********************************************************/
