/****
 *
 * Arquivo: ArvoreC.c
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 22/07/2012
 * ltima modificao: 22/07/2012
 *
 * Descrio: Implementao do mdulo ArvoreC
 *            (rvores costuradas)
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Funes de Entrada/Sada   */
#include <stdlib.h>   /* Funo exit()              */
#include "ArvoreC.h"  /* Declaraes do tipo rvore */
#include "ASSEGURA.h" /* Macro ASSEGURA             */

/***************** Definies de Funes ******************/

/****
 *
 * ConstroiNoC(): Constri um n de uma rvore binria
 *                costurrada
 *
 * Parmetros:
 *      item (entrada) - item de dados do n
 *
 * Retorno: Endereo do n criado
 *
 ****/
tArvoreC ConstroiNoC(tConteudo item)
{
   tNoC *pNo;

   pNo = malloc(sizeof(tNoC));

   ASSEGURA(pNo, "Erro: Nao foi possivel alocar no'");

   pNo->conteudo = item;
   pNo->esquerda = NULL;
   pNo->direita = NULL;

   return pNo;
}

/****
 *
 * FilhoEsquerdaC(): Cria o filho da esquerda de um n de uma
 *                   rvore binria costurada em ordem infixa
 *
 * Parmetros:
 *      pNo (entrada/sada) - ponteiro para o n cujo filho
 *                            ser criado
 *      item (entrada) - item de dados do filho
 *
 * Retorno: Nada
 *
 ****/
void FilhoEsquerdaC(tNoC *pNo, tConteudo item)
{
   tArvoreC q;

      /* O ponteiro para o n no pode ser NULL */
   ASSEGURA(pNo, "Erro: Insercao invalida.");

      /* O n no deve ter filho  esquerda ainda */
   ASSEGURA(!pNo->esquerda, "Erro: Insercao invalida.");

      /* Cria o filho da esquerda do n */
      /* p e faz q apontar para ele     */
   q = pNo->esquerda = ConstroiNoC(item);

      /* O filho da direita do novo n    */
      /* conter uma costura para seu pai */
   q->costura = SIM;
   q->direita = pNo;
}

/****
 *
 * FilhoDireitaC(): Cria o filho da direita de um n de uma
 *                  rvore binria costurada em ordem infixa
 *
 * Parmetros:
 *      pNo (entrada/sada) - ponteiro para o n cujo filho
 *                            ser criado
 *      item (entrada) - item de dados do filho
 *
 * Retorno: Nada
 *
 ****/
void FilhoDireitaC(tNoC *pNo, tConteudo item)
{
   tArvoreC q, r;

      /* O ponteiro para o n no pode ser NULL */
   ASSEGURA(pNo, "Erro: Insercao invalida.");

      /* O n no deve ter filho  direita */
   ASSEGURA( !pNo->direita || pNo->costura == SIM,
             "Erro: Insercao direita invalida." );

      /* Constri um novo n e faz q apontar para ele */
   q = ConstroiNoC(item);

      /* Guarda o sucessor em ordem infixa de p */
   r = pNo->direita;

      /* Torna o novo n filho  direita de p */
   pNo->direita = q;

      /* O ponteiro direito de p aponta para o  */
      /* novo n e, portanto, no  uma costura */
   pNo->costura = NAO;

      /* Se o sucessor em ordem infixa de p no  */
      /* for NULL, o novo n conter uma costura */
   q->costura = r ? SIM : NAO;

      /* O ponteiro direito do novo n apontar para */
      /* o sucessor em ordem infixa de seu pai       */
   q->direita = r;
}

/****
 *
 * InordemC(): Efetua um caminhamento em ordem infixa
 *             numa rvore binria costurada em ordem
 *             infixa  direita
 *
 * Parmetros:
 *      arvore (entrada) - ponteiro para a raiz da
 *                         rvore sobre a qual ser
 *                         efetuado o caminhamento
 *      op (entrada) - ponteiro para a funo que representa
 *                     a operao a ser efetuada sobre a
 *                     informao contida em cada n
 *
 * Retorno: Nada
 *
 ****/
void InordemC(tArvoreC arvore, tOperacao op)
{
   tArvoreC p,q;

   p = arvore;

   do {
      q = NULL;

      while (p) {
         q = p;
         p = p->esquerda;
      }

      if (q) {
         op(q->conteudo);

         p = q->direita;

         while (q->costura == SIM) {
            op(p->conteudo);
            q = p;
            p = q->direita;
         }
      }
   } while (q);
}

/****
 *
 * PreordemC(): Efetua um caminhamento em ordem prefixa numa
 *              rvore binria
 *
 * Parmetros:
 *      arvore (entrada) - ponteiro para a raiz da
 *                         rvore sobre a qual ser
 *                         efetuado o caminhamento
 *      op (entrada) - ponteiro para a funo que representa
 *                     a operao a ser efetuada sobre a
 *                     informao contida em cada n
 *
 * Retorno: Nada
 *
 * Observao: Esta implementao  semelhante quela do
 *             mdulo Arvore1
 *
 ****/
void PreordemC(tArvoreC arvore, tOperacao op)
{
   if (arvore) {
         /* Visita a raiz */
      op(arvore->conteudo);

         /* Caminha na subrvore da esquerda */
      PreordemC(arvore->esquerda, op);

         /* Caminha na subrvore da direita, se o */
         /* ponteiro direto no for uma costura   */
      if (arvore->costura == NAO) {
         PreordemC(arvore->direita, op);
      }
   }
}

/****
 *
 * PosordemC(): Efetua um caminhamento em ordem sufixa numa
 *              rvore binria
 *
 * Parmetros:
 *       arvore (entrada) - ponteiro para a raiz da
 *                          rvore sobre a qual ser
 *                          efetuado o caminhamento
 *       op (entrada) - ponteiro para a funo que representa
 *                      a operao a ser efetuada sobre a
 *                      informao contida em cada n
 *
 * Retorno: Nada
 *
 * Observao: Esta implementao  semelhante quela do
 *             mdulo Arvore1
 *
 ****/
void PosordemC(tArvoreC arvore, tOperacao op)
{
   if (arvore) {
         /* Caminha na subrvore esquerda */
      PosordemC(arvore->esquerda, op);

         /* Caminha na subrvore direita se o      */
         /* ponteiro  direita no for uma costura */
      if (arvore->costura == NAO) {
         PosordemC(arvore->direita, op);
      }

         /* Visita raiz */
      op(arvore->conteudo);
   }
}

/****
 *
 * ExibeNivelC(): Exibe na tela uma rvore por nvel
 *
 * Parmetros:
 *      arvore (entrada) - ponteiro para a raiz da
 *                         rvore que ser impressa
 *      fImpressao (entrada) - ponteiro para a funo que
 *                             efetua a escrita
 *      nivel (entrada) - nvel a partir do qual a rvore
 *                        ser impressa
 *
 * Retorno: Nada
 *
 ****/
static void ExibeNivelC( tArvoreC arvore,
                         tOperacao fImpressao, int nivel )
{
   int i;

   if (arvore) {
      for (i = 0; i < nivel; ++i) {
         printf("\t");
      }

      fImpressao(arvore->conteudo);

      printf("\n");

      if (arvore->costura == NAO) {
         ExibeNivelC( arvore->direita, fImpressao,
                        nivel + 1 );
      }

      ExibeNivelC(arvore->esquerda, fImpressao, nivel + 1);
   }
}

/****
 *
 * ExibeC(): Exibe na tela uma rvore binria costurada
 *            usando tabulaes (endentaes) para
 *            refletir a hierarquia entre os ns
 *
 * Parmetros:
 *      arvore (entrada) - ponteiro para a raiz da
 *                         rvore sobre a qual ser
 *                         efetuado o caminhamento
 *      op (entrada) - ponteiro para a funo que representa
 *                     a operao a ser efetuada sobre a
 *                     informao contida em cada n
 *
 * Retorno: Nada
 *
 * Observao: A funo que realmente faz o servio 
 *             ExibeNivelC().
 *
 ****/
void ExibeC(tArvoreC arvore, tOperacao fImpressao)
{
   ExibeNivelC(arvore, fImpressao, 0);
}
